#   Program:    versionInfo.py
#   Author:     Christopher Hanley
#
#   License: http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE
#
#   Date:       19 January 2004
#   Purpose:
#       To print a user's system information when providing user support.
#
#   Version:
#       Version 0.1.0, 19-Jan-04: Program created. -- CJH
#       Version 0.1.1, 20-Jan-04: Modified program to
#                                 loop over a taskList object. -- CJH
#
#       Version 0.2.0, 31-Mar-06: Added numpy to the task list. -- CJH
from __future__ import division, print_function # confidence high

__version__ = '0.2.0'

def printVersionInfo():
    # Print the current path information
    try:
        print("Path information:")
        print("-----------------")
        import sys
        print(sys.path)
        print(" ")
    except:
        print("Unable to get sys information.")
        print(" ")

    # Define the list of tasks to test
    taskList = [
                'numarray',
                'numpy',
                'Numeric',
                'pyfits',
                'pyraf',
                'multidrizzle',
                'pydrizzle',
                'stsci.tools',
                'calcos',
                'convolve',
                'image',
                'imagemanip',
                'imagestats',
                'ndimage'
                ]

    # Test the list of software tasks
    for software in taskList:
        print(software+":")
        print("-----------")
        try:
            package = __import__(software)
            try:
                print("version -> ",package.__version__)
            except:
                print("__version__ attribute is not defined")
            try:
                print("SVN version -> ",package.__svn_version__)
            except:
                print("__svn_version__ attribute is not defined")
            try:
                pathName = package.__path__
            except:
                pathName = package.__file__
            print("location -> ",pathName)
        except:
            print(software+" not found in path...")
        print(" ")

    # Print instruction message.
    print("PLEASE PASTE THE OUTPUT FROM THIS TASK ")
    print("INTO AN E-MAIL MESSAGE AND SEND IT WITH")
    print("YOUR PROBLEM DESCRIPTION TO SSB!")
    print(" ")
    print("SUPPORT ADDRESS: help@stsci.edu ")

if __name__ == '__main__':
    printVersionInfo()



"""

License: http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE

"""
from __future__ import absolute_import, division, generators # confidence medium

from . import numerixenv
numerixenv.check()

import numpy as N

BUFSIZE = 1024*1000   # 1Mb cache size

__version__ = '0.2'


def ImageIter(imgarr, bufsize=None, overlap=0, copy=0):

    imgarr = N.asarray(imgarr)

    if bufsize is None:
        bufsize = BUFSIZE

    if len(imgarr.shape) == 1:
        if copy:
            yield imgarr.copy()
        else:
            yield imgarr

    else:
        nrows = int(bufsize / (imgarr.itemsize * imgarr.shape[1]))
        niter = int(imgarr.shape[0] / nrows) * nrows

        if copy:
            # Create a cache that will contain a copy of the input
            # not just a view...
            _cache = N.zeros((nrows, imgarr.shape[1]), dtype=imgarr.dtype.char)

        for pix in range(0, niter+1, nrows):
            if copy:
                _cache = imgarr[pix:pix+nrows].copy()
                yield _cache
            else:
                yield imgarr[pix:pix+nrows]

            if copy:
                _cache *= 0

            pix -= overlap





# AUTOMATICALLY GENERATED BY 'RELIC':
# * DO NOT EDIT THIS MODULE MANUALLY.
# * DO NOT COMMIT THIS MODULE TO YOUR GIT REPOSITORY

__all__ = [
    '__version__',
    '__version_short__',
    '__version_long__',
    '__version_post__',
    '__version_commit__',
    '__version_date__',
    '__version_dirty__',
    '__build_date__',
    '__build_time__',
    '__build_status__'
]

__version__ = '3.4.12'
__version_short__ = '3.4.12'
__version_long__ = '3.4.12-0-gfd82437d'
__version_post__ = '0'
__version_commit__ = 'fd82437d'
__version_date__ = '2018-02-07 16:36:45 -0500'
__version_dirty__ = False
__build_date__ = '2018-02-14'
__build_time__ = '21:42:02.379382'
__build_status__ = 'release' if not int(__version_post__) > 0 \
    and not __version_dirty__ \
    else 'development'



"""
Return the gaussian fit of a 1D array.

Uses mpfit.py - a python implementation of the Levenberg-Marquardt
least-squares minimization, based on MINPACK-1. See nmpfit.py for
the history of this module (fortran -> idl -> python).
nmpfit.py is a version of mpfit.py which uses numarray.

@author: Nadia Dencheva
@version: '1.0 (2007-02-20)'

"""
from __future__ import absolute_import, division, print_function

__version__ = '1.0'          #Release version number only
__vdate__ = '2007-02-20'     #Date of this version

from . import numerixenv
numerixenv.check()

from . import nmpfit
import numpy as N
from numpy import random

def _gauss_funct(p, fjac = None, x = None, y=None, err=None,
weights=None):

    """
    Defines the gaussian function to be used as the model.

    """
    if p[2] != 0.0:
        Z = (x - p[1]) / p[2]
        model = p[0]*N.e ** (-Z**2 / 2.0)
    else:
        model = N.zeros(N.size(x))

    status = 0
    if weights is not None:
        if err is not None:
            print("Warning: Ignoring errors and using weights.\n")

        return [status, (y - model) * weights]

    elif err is not None:
        return [status, (y - model) / err]

    else:
        return [status, y-model]

def test_gaussfit():
    x=N.arange(10,20, 0.1)
    #x1=N.arange(0,10,0.1)
    #y1=5*N.e**(-(5-x1)**2/4)
    n=random.randn(100)
    y= 10*N.e**(-(15-x)**2/4) +n*3
    #x=N.arange(100, typecode=N.Int)
    #y=n.zeros(10, typecode=n.Float)
    #y= random.rand(100)
    #err = N.zeros(100)
    #return gaussfit(x,y, maxiter=20) #, x,y, n
    return gfit1d(y,x, maxiter=20)

def gfit1d(y, x=None, err = None, weights=None, par=None, parinfo=None,
           maxiter=200, quiet=0):
    """
    Return the gaussian fit as an object.

    Parameters
    ----------
    y:   1D Numarray array
        The data to be fitted
    x:   1D Numarray array
        (optional) The x values of the y array. x and y must
        have the same shape.
    err: 1D Numarray array
        (optional) 1D array with measurement errors, must be
        the same shape as y
    weights: 1D Numarray array
        (optiional) 1D array with weights, must be the same
        shape as y
    par:  List
        (optional) Starting values for the parameters to be fitted
    parinfo: Dictionary of lists
        (optional) provides additional information for the
        parameters. For a detailed description see nmpfit.py.
        Parinfo can be used to limit parameters or keep
        some of them fixed.
    maxiter: number
        Maximum number of iterations to perform
        Default: 200
    quiet: number
        if set to 1, nmpfit does not print to the screen
        Default: 0

    Examples
    --------
    >>> x=N.arange(10,20, 0.1)
    >>> y= 10*N.e**(-(x-15)**2/4)
    >>> print gfit1d(y,x=x, maxiter=20,quiet=1).params
    [ 10.          15.           1.41421356]

    """
    if numerixenv.check_input(x) or numerixenv.check_input(y):
        raise ValueError("Input is a NumArray array. This version of %s requires a Numpy array\n" % __name__)

    y = y.astype(N.float)
    if weights is not None:
        weights = weights.astype(N.float)
    if err is not None:
        err = err.astype(N.float)
    if x is None and len(y.shape)==1 :
        x = N.arange(len(y)).astype(N.float)
    if x.shape != y.shape:
        print("input arrays X and Y must be of equal shape.\n")
        return


    fa = {'x':x, 'y':y, 'err':err, 'weights':weights}

    if par is not None:
        p = par
    else:
        ysigma = y.std()
        ind = N.nonzero(y > ysigma)[0]
        if len(ind) != 0:
            xind = int(ind.mean())
            p2 = x[xind]
            p1 = y[xind]
            p3 = 1.0
        else:
            ymax = y.max()
            ymin = y.min()
            ymean= y.mean()
            if (ymax - ymean) > (abs(ymin - ymean)):
                p1 = ymax
            else: p1 = ymin
            ind = (N.nonzero(y == p1))[0]
            p2 = x.mean()
            p3 = 1.


        p = [p1, p2, p3]
    m=nmpfit.mpfit(_gauss_funct, p,parinfo = parinfo, functkw=fa,
maxiter=maxiter, quiet=quiet)
    if (m.status <=0): print('error message = ', m.errmsg)
    return m


def plot_fit(y, mfit, x=None):
    if x is None:
        x = N.arange(len(y))
    else:
        x = x
    p = mfit.params
    #y = gauss_funct(p, y)
    yy = p[0] + N.e**(-0.5*(x-p[1])**2/p[2]**2)
    try:
        import pylab
    except ImportError:
        print("Matplotlib is not available.\n")
        return
    pylab.plot(x,yy)

def test():
    import doctest
    from . import gfit
    return doctest.testmod(gfit)



from __future__ import division, print_function
import math, os, sys, time
import unittest
import numpy as N
""" This module extends the built-in unittest capabilities to facilitate
performing floating point comparisons on scalars and numpy arrays. It also
provides functions that automate building a test suite from all tests
present in the module, and running the tests in standard or debug mode.

To use this module, import it along with unittest [QUESTION: should this
module import everything from unittest into its namespace to make life
even easier?]. Subclass test cases from testutil.FPTestCase instead of
unittest.TestCase. Call testall or debug from this module:

import testutil

class FileTestCase(testutil.FPTestCase):
    def setUp(self):
        assorted_test_setup

    def testone(self):
        self.assertEqual(1,2)

    def testtwo(self):
        self.assertApproxNumpy(arr1,arr2,accuracy=1e-6)

    def tearDown(self):
        assorted_test_teardown

if __name__ == '__main__':
    if 'debug' in sys.argv:
        testutil.debug(__name__)
    else:
        testutil.testall(__name__,2)

To run the tests in normal mode from the shell, then do the following:
    python my_module.py
It will run all tests, success or failure, and print a summary of the results.

To run the tests in debug mode from the shell, do the following:
    python -i my_module.py debug
    >>> import pdb
    >>> pdb.pm()
In debug mode, it will run until it encounters the first failed test, then
stop. Thus if you run with the -i switch, you can then import pdb and
proceed with the usual debugger commands.

If you prefer to run your tests from within the python interpreter,
you may import this module and call its testall() and debug() functions
explicitly. The modules you are testing must be visible in your sys.path.

>>>import testutil as U
>>> U.testall('ui_test')

"""

class LogTestCase(unittest.TestCase):
   """Override the .run() method to do some logging"""
   def run(self, result=None):
        if result is None: result = self.defaultTestResult()
        result.startTest(self)
        testMethod = getattr(self, self._testMethodName)
        try:
            try:
                self.setUp()
            except KeyboardInterrupt:
                raise
            except:
                result.addError(self, self._exc_info())
                self.log('E')
                return

            ok = False
            try:
                testMethod()
                ok = True
                self.log("P")
            except self.failureException:
                result.addFailure(self, self._exc_info())
                self.log("F")
            except KeyboardInterrupt:
                raise
            except:
                result.addError(self, self._exc_info())
                self.log("E")

            try:
                self.tearDown()
            except KeyboardInterrupt:
                raise
            except:
                result.addError(self, self._exc_info())
                ok = False
            if ok: result.addSuccess(self)
        finally:
            result.stopTest(self)

   def log(self,status,name=None):
      """Creates a log file containing the test name, status,and timestamp,
      as well as any attributes in the tda and tra dictionaries if present.
      Does not yet support fancy separating of multi-line items."""
      if name is None:
         try:
            name=self.name
         except AttributeError:
            name=self.id()
      try:
         f=open(name+'.log','w')
      except IOError as e:
         print("Error opening log file: %s"%e.strerror)
         print("***No Logging Performed***")
         return

      f.write("%s:: Name=%s\n"%(name,name))
      f.write("%s:: Status=%s\n"%(name,status))
      f.write("%s:: Time=%s\n"%(name,time.asctime()))
      try:
         for k in self.tda:
            f.write("%s:: tda_%s=%s\n"%(name,str(k),str(self.tda[k])))
      except AttributeError:
         pass

      try:
         for k in self.tra:
            f.write("%s:: tra_%s=%s\n"%(name,str(k),str(self.tra[k])))
      except AttributeError:
         pass

      if status == 'E':
          f.write("%s:: tra_Trace=%s\n"%(name,str(self._exc_info())))

      f.write("END\n")
      f.close()





class FPTestCase(unittest.TestCase):
    ''' Base class to hold some functionality related to floating-point
    precision and array comparisons'''

    def assertApproxFP(self, testvalue, expected, accuracy=1.0e-5):
        ''' Floating point comparison  '''
        result = math.fabs((testvalue - expected) / expected)
        self.failUnless(result <= accuracy,"test: %g, ref: %g"%(testvalue,expected))

    def assertApproxNumpy(self, testarray, expected, accuracy=1.0e-5):
        ''' Floating point array comparison '''
        result=N.abs(testarray-expected)/expected
        self.failUnless(N.alltrue(result <= accuracy))

    def assertEqualNumpy(self, testarray, expected):
        ''' Identical FP array comparison '''
        self.failUnless(N.alltrue(testarray == expected))

class LogTextRunner(unittest.TextTestRunner):
    """ Redefines the .run() method to call a .log() method on the test
    when it is complete. """

    def run(self, test):
        "Run the given test case or test suite."
        result = self._makeResult()
        startTime = time.time()
        test(result)
        stopTime = time.time()
        timeTaken = stopTime - startTime
        result.printErrors()
        self.stream.writeln(result.separator2)
        run = result.testsRun
        self.stream.writeln("Ran %d test%s in %.3fs" %
                            (run, run != 1 and "s" or "", timeTaken))
        self.stream.writeln()
        if not result.wasSuccessful():
            self.stream.write("FAILED (")
            failed, errored = list(map(len, (result.failures, result.errors)))
            if failed:
                self.stream.write("failures=%d" % failed)
                test.log("F")
            if errored:
                if failed: self.stream.write(", ")
                self.stream.write("errors=%d" % errored)
                test.log("E")
            self.stream.writeln(")")
        else:
            self.stream.writeln("OK")
            test.log("P")

        return result

def buildsuite(module):
    """Builds a test suite containing all tests found in the module.
    Returns the suite."""
    M = __import__(module)
    suite = unittest.defaultTestLoader.loadTestsFromModule(M)
    return suite

def debug(module):
    """ Build the test suite, then run in debug mode, which allows for postmortems"""
    buildsuite(module).debug()

def testall(module,verb=0):
    """ Build and run the suite through the testrunner. Verbosity level
    defaults to quiet but can be set to 2 to produce a line as it runs
    each test. A summary of the number of tests run, errors, and failures
    is always printed at the end."""
    result=unittest.TextTestRunner(verbosity=verb).run(buildsuite(module))
    return result

def testlog(module,verb=0):
    result=LogTextRunner(verbosity=verb).run(buildsuite(module))
    return result

def dump_file(fname, hdrwidth=80):
    """ Convenience function to dump a named file to the stdout, with
    an optional header listing the filename.  This is easy to do without
    a convenience function like this, but having one reduces code in the XML
    test files. """
    assert os.path.exists(fname), "dump_file could not find: "+fname
    sys.stdout.flush()
    if hdrwidth>0:
        print("")
        print("="*hdrwidth)
        print(fname+':')
        print("="*hdrwidth)
    f = open(fname, 'r')
    for line in f:
        print(line.rstrip())
    f.close()

def dump_all_log_files(hdrwidth=80):
    """ Convenience function to dump all *.log files in cwd to the stdout,
    with an optional header listing each filename. See dump_file. """
    import glob
    flist = glob.glob('*.log')
    for f in flist:
        dump_file(f, hdrwidth=hdrwidth)



from __future__ import division # confidence medium
import os

def check_input(xxx):
    """Check if input is a Numarray Array."""
    try:
        import numarray
        return isinstance(xxx,numarray.numarraycore.NumArray)    
    except ImportError:
        pass

def check():
    """Check for running numarray version of pyfits with numpy code."""
    pass

    




from __future__ import division # confidence high

from stsci.tools.xyinterp import xyinterp
import numpy as N

x=N.array((1,2,3,4,5))
y=x.copy()

def test_xyinterp_1():
    #test 1
    ans = xyinterp(x,y,3)
    assert ans == 3, "Test 1 failed, ans = %f, should be 3"%ans

def test_xyinterp_2():
    #test 2
    ans = xyinterp(x,y,3.5)
    assert ans == 3.5, "Test 2 failed, ans = %f, should be 3.5"%ans

def test_xyinterp_3():
    #test 3
    try:
        ans = xyinterp(x,y,-3)
        raise AssertionError( "Test 3 failed; should have thrown an exception, answer = %s" % str(ans))
    except ValueError:
        pass

def test_xyinterp_4():
    #test 4
    try:
        ans = xyinterp(x,y,5.6)
        raise AssertionError( "Test 4 failed; should have thrown an exception, answer = %s" % str(ans))
    except ValueError:
        pass
    
def test_xyinterp_5():
    #test 5
    x=N.array((1,3,7,9,12))
    y=N.array((5,10,15,20,25))
    ans = xyinterp(x,y,8)
    assert ans == 17.5, "Test 5 failed, ans = %f, should be 17.5"%ans

def test_xyinterp_6():
    #test 6
    x=N.array((5,3,6,2,7,0))
    y=N.array((4,6,2,4,6,2))
    try:
        ans = xyinterp(x,y,2)
        raise AssertionError( "Test 6 failed; should have thrown an exception, answer = %s" % str(ans))
    except ValueError:
        pass

def test_xyinterp_7():
    #test 7
    x=N.array((1,2,3,4,5))
    y=N.arange(20)
    
    try:
        ans = xyinterp(x,y,2)
        raise AssertionError( "Test 7 failed; should have thrown an exception, answer = %s" % str(ans))
    except ValueError:
        pass

if __name__ == '__main__':
    test_xyinterp_1()
    test_xyinterp_2()
    test_xyinterp_3()
    test_xyinterp_4()
    test_xyinterp_5()
    test_xyinterp_6()
    test_xyinterp_7()



"""

License: http://www.stsci.edu/resources/software_hardware/pyraf/LICENSE

"""
from __future__ import division # confidence medium
from __future__ import generators

import types
import numpy as N


BUFSIZE = 1024*1000   # 1Mb cache size

__version__ = '0.7'
__vdate__ = '25-July-2012'


def ImageIter(imglist,bufsize=BUFSIZE,overlap=0,copy=0,updateSection = None):
    """ Return image section for each image listed on input.
        The inputs can either be a single image or a list of them,
        with the return value matching the input type.
        All images in a list MUST have the same shape, though,
        in order for the iterator to scroll through them properly.

        The size of section gets defined by 'bufsize', while
        'copy' specifies whether to return an explicit copy
        of each input section or simply return views.
        The 'overlap' parameter provides a way of scrolling
        through the image with this many rows of overlap, with
        the default being no overlap at all.
    """
    if type(imglist) != list:
        imgarr = imglist.data
        imgarr = N.asarray(imgarr)
        _imglen = 1
        single = 1
    else:
        imgarr = imglist[0].data
        imgarr = N.asarray(imgarr)
        _imglen = len(imglist)
        single = 0
        _outlist = []
    _numrows = imgarr.shape[0]

    if len(imgarr.shape) == 1:
        if copy:
            if single:
                yield imgarr.copy(),None
            else:
                for img in imglist: _outlist.append(img.copy())
        else:
            yield imglist,None

    else:
        nrows = computeBuffRows(imgarr,bufsize=bufsize)
#       niter = int(imgarr.shape[0] / nrows) * nrows
        nbuff,nrows = computeNumberBuff(imgarr.shape[0],nrows,overlap)
        niter = nbuff*nrows

        if copy:
            # Create a cache that will contain a copy of the input
                    # not just a view...
            if single:
                _cache = N.zeros((nrows,imgarr.shape[1]),dtype=imgarr.dtype)
            else:
                for img in imglist: _outlist.append(N.zeros((nrows,imgarr.shape[1]),dtype=imgarr.dtype))

        for pix in range(0,niter+1,nrows):
            # overlap needs to be computed here
            # This allows the user to avoid edge effects when
            # convolving the returned image sections, and insures
            # that the last segment will always be returned with
            # overlap+1 rows.

            _prange = pix+nrows+overlap
            if _prange > _numrows: _prange = _numrows
            if pix == _prange: break

            if copy:
                if single:
                    _cache = imgarr[pix:_prange].copy()
                    yield _cache,(pix,_prange)
                    N.multiply(_cache,0.,_cache)
                else:
                    for img in range(len(imglist)): _outlist[img] = imglist[img][pix:_prange].copy()
                    yield _outlist,(pix,_prange)
                    for img in range(len(imglist)): N.multiply(_outlist[img],0.,_outlist[img])
            else:
                if single:
                    #yield imgarr.section[pix:_prange,:],(pix,_prange)
                    yield imgarr[pix:_prange],(pix,_prange)
                else:
                    for hdu in imglist:
                        #_outlist.append(imglist[img][pix:pix+nrows])
                        _outlist.append(hdu.section[pix:_prange,:])
                    yield _outlist,(pix,_prange)
                    # This code is inserted to copy any values changed
                    # in the image sections back into the original image.
                    if updateSection is not None:
                        #for _index in xrange(len(_outlist)):
                        imglist[updateSection][pix:_prange] = _outlist[updateSection]
                    del _outlist
                    _outlist = []

def computeBuffRows(imgarr,bufsize=BUFSIZE):
    """ Function to compute the number of rows from the
        input array that fits in the allocated memory given
        by the bufsize.
    """
    imgarr = N.asarray(imgarr)
    buffrows = int(bufsize / (imgarr.itemsize * imgarr.shape[1]))
    return buffrows

def computeNumberBuff(numrows, buffrows, overlap):
    """ Function to compute the number of buffer sections
        that will be used to read the input image given the
        specified overlap.
    """
    nbuff = _computeNbuff(numrows, buffrows, overlap)
    niter = 1 + int(nbuff)
    totalrows = niter * buffrows
    # We need to account for the case where the number of
    # iterations ends up being greater than needed due to the
    # overlap.
    #if totalrows > numrows: niter -= 1
    lastbuff = numrows - (niter*(buffrows-overlap))

    if lastbuff < overlap+1 and nbuff > 1:
        good = False
        while not good:
            if buffrows > overlap+1:
                buffrows -= 1

                nbuff = _computeNbuff(numrows, buffrows, overlap)
                niter = 1 + int(nbuff)
                totalrows = niter * (buffrows - overlap)
                lastbuff = numrows - (niter*(buffrows-overlap))
                if lastbuff > overlap + 1:
                    good = True
            else:
                good = True
    return niter,buffrows

def _computeNbuff(numrows,buffrows,overlap):

    if buffrows > numrows:
        nbuff = 1
    else:
        overlaprows = buffrows - overlap
        rowratio = (numrows - overlaprows)/(1.0*buffrows)
        nbuff = (numrows - overlaprows+1)/(1.0*overlaprows)
    return nbuff

def FileIter(filelist,bufsize=BUFSIZE,overlap=0):
    """ Return image section for each image listed on input, with
        the object performing the file I/O upon each call to the
        iterator.

        The inputs can either be a single image or a list of them,
        with the return value matching the input type.
        All images in a list MUST have the same shape, though,
        in order for the iterator to scroll through them properly.

        The size of section gets defined by 'bufsize'.
        The 'overlap' parameter provides a way of scrolling
        through the image with this many rows of overlap, with
        the default being no overlap at all.
    """
    if type(filelist) != list:
        imgarr = filelist.data
        imgarr = N.asarray(imgarr)
        _imglen = 1
        single = 1
    else:
        imgarr = filelist[0].data
        imgarr = N.asarray(imgarr)
        _imglen = len(filelist)
        single = 0
        _outlist = []
    _numrows = imgarr.shape[0]

    if len(imgarr.shape) == 1:
        # This needs to be generalized to return pixel ranges
        # based on bufsize, just like with 2-D arrays (images).
        yield filelist,None

    else:
        nrows = computeBuffRows(imgarr,bufsize=bufsize)
#       niter = int(imgarr.shape[0] / nrows) * nrows
        nbuff,nrows = computeNumberBuff(imgarr.shape[0],nrows,overlap)
        niter = nbuff * nrows

        for pix in range(0,niter+1,nrows-overlap):
            # overlap needs to be computed here
            # This allows the user to avoid edge effects when
            # convolving the returned image sections, and insures
            # that the last segment will always be returned with
            # overlap+1 rows.
            _prange = pix+nrows
            if _prange > _numrows: _prange = _numrows
            if pix >= _prange: break
            if single:
                yield imgarr[pix:_prange],(pix,_prange)
            else:
                for hdu in filelist:
                    _outlist.append(hdu[pix:_prange])
                yield _outlist,(pix,_prange)
                del _outlist
                _outlist = []



#!/usr/bin/env python
#
"""
   This module is from Lennart Regebro's ComparableMixin class, available at:

       http://regebro.wordpress.com/2010/12/13/
              python-implementing-rich-comparison-the-correct-way/

   The idea is to prevent you from having to define lt,le,eq,ne,etc...
   This may no longer be necessary after the functools total_ordering
   decorator (Python v2.7) is available on all Python versions
   supported by our software.

   For simple comparisons, all that is necessary is to derive your class
   from ComparableMixin and override the _cmpkey() method.

   For more complex comparisons (where type-checking needs to occur and
   comparisons to other types are allowed), simply override _compare() instead
   of _cmpkey().

   BEWARE that comparing different types has different results in Python 2.x
   versus Python 3.x:

        Python 2.7
        >>> 'a' < 2
        False

        Python 3.2.1
        >>> 'a' < 2
        Traceback (most recent call last):
          File "<stdin>", line 1, in <module>
        TypeError: unorderable types: str() < int()
"""
from __future__ import print_function

import sys
if sys.version_info[0] < 3:
    string_types = basestring
else:
    string_types = str

class ComparableMixin(object):
    def _compare(self, other, method):
        try:
            return method(self._cmpkey(), other._cmpkey())
        except (AttributeError, TypeError):
            # _cmpkey not implemented, or return different type,
            # so I can't compare with "other".
            return NotImplemented

    def __lt__(self, other):
        return self._compare(other, lambda s,o: s < o)

    def __le__(self, other):
        return self._compare(other, lambda s,o: s <= o)

    def __eq__(self, other):
        return self._compare(other, lambda s,o: s == o)

    def __ge__(self, other):
        return self._compare(other, lambda s,o: s >= o)

    def __gt__(self, other):
        return self._compare(other, lambda s,o: s > o)

    def __ne__(self, other):
        return self._compare(other, lambda s,o: s != o)


class ComparableIntBaseMixin(ComparableMixin):
    """ For those classes which, at heart, are comparable to integers. """
    def _compare(self, other, method):
        if isinstance(other, self.__class__): # two objects of same class
            return method(self._cmpkey(), other._cmpkey())
        else:
            return method(int(self._cmpkey()), int(other))


class ComparableFloatBaseMixin(ComparableMixin):
    """ For those classes which, at heart, are comparable to floats. """
    def _compare(self, other, method):
        if isinstance(other, self.__class__): # two objects of same class
            return method(self._cmpkey(), other._cmpkey())
        else:
            return method(float(self._cmpkey()), float(other))


# -----------------------------------------------------------------------------


# this class is only used for testing this module!
class SimpleStrUnitTest(ComparableMixin):
    def __init__(self, v): self.val = str(v) # all input turned to string
    def __str__(self): return str(self.val)
    def _cmpkey(self): return self.val


# this class is only used for testing this module!
class AnyTypeUnitTest(ComparableMixin):
    def __init__(self, v): self.val = v # leave all input typed as is
    def __str__(self): return str(self.val)

    # define this instead of _cmpkey - handle ALL sorts of scenarios,
    # except intentionally don't compare self strings (strlen>1) with integers
    # so we have a case which fails in our test below
    def _compare(self, other, method):
        if isinstance(other, self.__class__):
            return self._compare(other.val, method) # recurse, get 2 logic below
        if isinstance(other, string_types):
            return method(str(self.val), other)
        elif other is None and self.val is None:
            return method(0, 0)
        elif other is None:
            return method(str(self.val), '') # coerce to str compare
        elif isinstance(other, int):
            # handle ONLY case where self.val is a single char or an int
            if isinstance(self.val, string_types) and len(self.val)==1:
                return method(ord(self.val), other)
            else:
                return method(int(self.val), other) # assume we are int-like
        try:
            return method(self.val, other)
        except (AttributeError, TypeError):
            return NotImplemented


# -----------------------------------------------------------------------------


def test():
    a = SimpleStrUnitTest('a')
    b = SimpleStrUnitTest('b')
    c = SimpleStrUnitTest('c')
    two = SimpleStrUnitTest(2)

    # compare two SimpleStrUnitTest objects
    assert str(a>b) == "False"
    assert str(a<b) == "True"
    assert str(a<=b) == "True"
    assert str(a==b) == "False"
    assert str(b==b) == "True"
    assert str(a<c) == "True"
    assert str(a<=c) == "True"
    assert str(a!=c) == "True"
    assert str(c!=c) == "False"
    assert str(c==c) == "True"
    assert str(b<two) == "False"
    assert str(b>=two) == "True"
    assert str(b==two) == "False"
    assert str([str(jj) for jj in sorted([b,a,two,c])])=="['2', 'a', 'b', 'c']"
    print('Success in first set')

    x = AnyTypeUnitTest('x')
    y = AnyTypeUnitTest('yyy')
    z = AnyTypeUnitTest(0)
    nn = AnyTypeUnitTest(None)

    # compare two AnyTypeUnitTest objects
    assert str(x>y) == "False"
    assert str(x<y) == "True"
    assert str(x<=y) == "True"
    assert str(x==y) == "False"
    assert str(y==y) == "True"
    assert str(x<z) == "False"
    assert str(x<=z) == "False"
    assert str(x>z) == "True"
    assert str(x!=z) == "True"
    assert str(z!=z) == "False"
    assert str(z==z) == "True"
    assert str(y<nn) == "False"
    assert str(y>=nn) == "True"
    assert str(y==nn) == "False"
    assert str(nn==nn) == "True"
    assert str([str(jj) for jj in sorted([y,x,nn,z])]) == "['None', '0', 'x', 'yyy']"
    print('Success in second set')

    # compare AnyTypeUnitTest objects to built-in types
    assert str(x<0) == "False"
    assert str(x<=0) == "False"
    assert str(x>0) == "True"
    assert str(x!=0) == "True"
    assert str(x==0) == "False"
    assert str(x<None) == "False"
    assert str(x<=None) == "False"
    assert str(x>None) == "True"
    assert str(x!=None) == "True"
    assert str(x==None) == "False"
    assert str(x<"abc") == "False"
    assert str(x<="abc") == "False"
    assert str(x>"abc") == "True"
    assert str(x!="abc") == "True"
    assert str(x=="abc") == "False"
    assert str(y<None) == "False"
    assert str(y<=None) == "False"
    assert str(y>None) == "True"
    assert str(y!=None) == "True"
    assert str(y==None) == "False"
    assert str(y<"abc") == "False"
    assert str(y<="abc") == "False"
    assert str(y>"abc") == "True"
    assert str(y!="abc") == "True"
    assert str(y=="abc") == "False"
    print('Success in third set')

    # all of the above should work without errors; now raise some
    print('yyy == 0 ?')
    try:
        y == z # AnyTypeUnitTest intentionally doesn't compare strlen>1 to ints
        assert 0, 'Exception expected but not found'
    except ValueError:
        print('   ... exception handled')

    print('sorted([0, yyy]) ?')
    try:
        sorted([z,y])
        assert 0, 'Exception expected but not found'
    except ValueError:
        print('   ... exception handled')
    print('Test successful')

# -----------------------------------------------------------------------------

if __name__=='__main__': # in case something else imports this file
    test()



#!/usr/bin/env python

"""
Package: stsci.tools
Author: Christopher Hanley

Purpose:
========
Provide driver function for package tests.

Dependencies:
=============

- nose 0.10.4 or greater.

Usage Example:
==============
All packages will need to import stsci.tools.tester and add the following
function to the __init__.py of their package:

import stsci.tools.tester
def test(*args,**kwds):
    return stsci.tools.tester.test(modname=__name__, *args, **kwds)


This assumes that all software packages are installed with the structure:

package/
    __init__.py
    modules.py
    tests/
    tests/__init__.py
    tests/test_whatever.py

Where the /tests subdirectory containts the python files that nose will
recognize as tests.

"""

from __future__ import division, print_function

import os
import os.path
import sys

pytools_tester_active = False

def test(modname, mode='nose', *args, **kwds):
    """
    Purpose:
    ========
    test: Run refcore nosetest suite of tests. The tests are located in the
    tests/ directory of the installed modules.

    """

    global pytools_tester_active

    if modname is not None :
        curdir = sys.modules[modname].__file__
        curdir = os.path.abspath(curdir)
        curdir = os.path.dirname(curdir)
    else:
        raise ValueError('name of module to test not given')

    DIRS = [os.path.join(curdir, testdir) for testdir in ['tests', 'test']]

    dirname = None
    for x in DIRS:
        if os.path.isdir(x) :
            dirname = x
            break

    if dirname is None :
            print('no tests found in: %s' % repr(DIRS))
            return False

    if mode == 'nose' :

        print("Testing with nose in %s\n"%dirname)
        try:
            import nose
        except ImportError:
            print("Nose 0.10.4 or greater is required for running tests.")
            raise

        # First arg is blank, since it's skipped by nose
        # --exe is needed because easy_install sets all .py files as executable for
        # some reason
        args = ['', '--exe', '-w', dirname ]

        result = False

        try :
            pytools_tester_active = True
            result = nose.run(argv=args)
        except :
            pytools_tester_active = False
            raise
        pytools_tester_active = False

        return result

    if mode == 'pytest' :

        print("Testing with pytest in %s\n"%dirname)

        try :
            import pytest
        except ImportError :
            print("py.test is required for running tests")
            raise

        # do not use --doctest-modules ; it doesn't work right
        args = [ dirname ]

        try :
            import pandokia
            args = ['-p', 'pandokia.helpers.pytest_plugin' ] + args
        except ImportError :
            pass

        result = False

        try :
            pytools_tester_active = True
            result = pytest.main(args)
        except :
            pytools_tester_active = False
            raise
        pytools_tester_active = False

        return result

    raise ValueError("invalid test specification - mode must be one of 'nose' or 'pytest'")




'''This is a tool for stashing certain information used by the
continuous integration system at STScI.  It is not intended for,
or even expected to work, in any other application.

--

use this in shell scripts:
    d=`python -m stsci.tools.stash`
    cp file $d

'''

from __future__ import print_function
import sys
import os

# use os.path.join because the file name may be used outside of
# python and we need it to be right on Windows.
stash_dir = os.path.join(os.path.dirname(__file__),'stash')

try :
    os.mkdir(stash_dir)
except OSError :
    pass

if __name__ == '__main__' :
    print(stash_dir)
    if not os.path.exists(stash_dir) :
        sys.exit(1)

